import { Component, inject, Input } from '@angular/core';
import { AbstractControl, FormArray, FormControl, FormGroup, ReactiveFormsModule, Validators } from "@angular/forms";
import { Hotel } from "../hotel-item/hotel";
import { HttpClient } from '@angular/common/http';
import { catchError, debounceTime } from "rxjs";
import { Router, RouterLink } from "@angular/router";
import { NgClass, NgFor, NgIf } from "@angular/common";
import { MatButtonModule } from "@angular/material/button";
import { MatIconModule } from "@angular/material/icon";
import { FormsModule } from '@angular/forms';

@Component({
  selector: 'app-hotel-form',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    RouterLink,
    NgIf,
    MatButtonModule,
    MatIconModule,
    NgFor,
    FormsModule,
    NgClass
  ],
  templateUrl: './hotel-form.component.html',
  styleUrl: './hotel-form.component.css'
})
export class HotelFormComponent {
  public form!: FormGroup;

  @Input()
  public hotel!: Hotel;

  public router: Router = inject(Router);

  public showDeleteConfirmation = false;

  constructor(private http: HttpClient) { }

  public errorMessage: string = '';

  ngOnInit() {
    this.form = new FormGroup({
      name: new FormControl(this.hotel?.hotelName || '', [Validators.minLength(3), Validators.required]),
      description: new FormControl(this.hotel?.description || '', [Validators.minLength(3), Validators.required]),
      price: new FormControl(this.hotel?.price || '', [Validators.min(0), Validators.required]),
      rating: new FormControl(this.hotel?.rating || '', [Validators.min(0), Validators.max(5), Validators.required]),
      tags: new FormArray(this.hotel?.tags?.map(tag => new FormControl(tag)) || [])
    });

    Object.keys(this.form.controls).forEach(key => {
      const control = this.form.get(key);
      control?.valueChanges.pipe(
        debounceTime(1000)
      ).subscribe(() => { 
        this.setError(control as AbstractControl, key);
      });
    });
  }

  public delete() {
    this.http.delete(`api/hotels/${this.hotel?.id}`).pipe(
      catchError(err => {
        console.error('Error deleting hotel', err);
        return err;
      })
    ).subscribe(() => {
      console.log('Hotel deleted');
    });

    this.router.navigate(['/']);
  }

  get tags() {
    return this.form.get('tags') as FormArray;
  }

  public addTag() {
    this.tags.push(new FormControl(''));
  }

  public submit() {
    if (this.form.valid && !this.errorMessage) {
      const hotelData: Hotel = {
        id: this.hotel?.id || 0,
        hotelName: this.form.value.name,
        description: this.form.value.description,
        price: this.form.value.price,
        rating: this.form.value.rating,
        imageUrl: this.hotel?.imageUrl || 'https://placehold.co/2000x1050/EEE/31343C',
        tags: this.form.value.tags || []
      };

      if (this.hotel) {
        this.http.put(`api/hotels/${this.hotel.id}`, hotelData).pipe(
          catchError(err => {
            console.error('Error updating hotel', err);
            return err;
          })
        ).subscribe(() => {
          console.log('Hotel updated');
        });
      }

      else {
        this.http.post('api/hotels', hotelData).pipe(
          catchError(err => {
            console.error('Error creating hotel', err);
            return err;
          })
        ).subscribe(() => {
          console.log('Hotel created');
        });
      }

      this.form.reset();
      this.router.navigate(['/']);
    }
  }

  private validationErrors: Record<string, string> = {
    'required': 'This field is required',
    'minlength': 'This field must be at least 3 characters long',
    'min': 'This field must be at least 0',
    'max': 'This field must be at most 5',
    'name.required': 'Hotel name is required',
    'name.minlength': 'Hotel name must be at least 3 characters long',
    'description.required': 'Hotel description is required', 
    'description.minlength': 'Hotel description must be at least 3 characters long',
    'price.required': 'Hotel price is required',
    'price.min': 'Hotel price cannot be negative',
    'rating.required': 'Hotel rating is required',
    'rating.min': 'Hotel rating must be at least 0',
    'rating.max': 'Hotel rating cannot be more than 5',
    'tags.required': 'At least one tag is required',
  };

  private setError(control: AbstractControl, controlName: string) {
    if ((control.touched || control.dirty) && control.errors) {
      const errors = Object.keys(control.errors).map(key => {
        const errorKey = `${controlName}.${key}`;
        return this.validationErrors[errorKey] || this.validationErrors[key];
      });
      this.errorMessage = errors.join(' ');
    } else {
      this.errorMessage = '';
    }
  }
}

